pub mod codec;
pub mod constants;
pub mod de;
pub mod decoder;
pub mod encoder;
pub mod reader;
pub mod ser;
pub mod value;
pub mod writer;

pub use codec::Codec;
pub use de::Deserializer;
pub use de::from_value;
pub use decoder::Decoder;
pub use encoder::EncodePlaceholderMap;
pub use encoder::Encoder;
pub use reader::BinaryReader;
pub use reader::DecodePlaceholderMap;
pub use reader::Reader;
pub use reader::is_eof_error;
pub use reader::is_syntax_error;
pub use ser::Serializer;
pub use ser::to_value;
pub use value::Domain;
pub use value::NestedValue;
pub use value::Value;
pub use value::AnnotatedValue;
pub use value::PlainValue;
pub use value::RcValue;
pub use value::ArcValue;
pub use value::Set;
pub use value::Map;
pub use writer::Writer;

pub fn invert_map<A, B>(m: &Map<A, B>) -> Map<B, A>
    where A: Clone, B: Clone + Ord
{
    use std::iter::FromIterator;
    Map::from_iter(m.iter().map(|(a, b)| (b.clone(), a.clone())))
}

#[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub enum NullDomain {}
impl Domain for NullDomain {}
